@@ -4,9 +4,8 @@
 #' \code{psfmi_coxr} Pooling and backward selection for Cox regression
 #' models in multiply imputed datasets using different selection methods.
 #'
-#' @param data Data frame or data matrix with stacked multiple imputed
-#' datasets.
-#'   The original dataset that contains missing values must be excluded from the dataset. The imputed
+#' @param data Data frame with stacked multiple imputed
+#' datasets. The original dataset that contains missing values must be excluded from the dataset. The imputed
 #'   datasets must be distinguished by an imputation variable, specified under impvar, and starting by 1.
 #' @param nimp A numerical scalar. Number of imputed datasets. Default is 5.
 #' @param impvar A character vector. Name of the variable that distinguishes the imputed datasets.
@@ -40,11 +39,16 @@
 #'  by using the rcs function for restricted cubic splines of the rms package of Frank Harrell.
 #'  A minimum number of 3 knots as defined under knots is needed.
 #'
-#'@return A \code{psfmi_coxr} object from which the following objects can be extracted: pooled model as 
-#'  \code{RR_model}, pooled p-values according to pooling method as \code{multiparm_p}, predictors 
-#'  excluded at each step as \code{coef.excl_step}, and \code{impvar}, \code{nimp}, \code{method}, 
-#'  \code{p.crit}, \code{predictors}, \code{cat.predictors}, \code{keep.predictors}, \code{int.predictors},
-#'  \code{spline.predictors}, \code{knots}, \code{print.method}.
+#'@return An object of class \code{smodsmi} (selected models in multiply imputed datasets) from 
+#'  which the following objects can be extracted: imputed datasets as \code{data}, selected 
+#'  pooled model as \code{RR_model}, pooled p-values according to pooling method as \code{multiparm}, 
+#'  predictors included at each selection step as \code{predictors_in}, predictors excluded at each step 
+#'  as \code{predictors_out}, and \code{impvar}, \code{nimp}, \code{time}, \code{status}, \code{method}, 
+#'  \code{p.crit}, \code{predictors}, \code{cat.predictors}, \code{keep.predictors}, \code{int.predictors}, 
+#'  \code{spline.predictors}, \code{knots}, \code{print.method}, \code{call}, \code{model_type} and 
+#'  \code{predictors_final} for names of predictors in final selection step, \code{fit.formula} is the 
+#'  regression formula of start model and \code{predictors_initial} for names of predictors in start
+#'  model.
 #'    
 #' @references Eekhout I, van de Wiel MA, Heymans MW. Methods for significance testing of categorical
 #'   covariates in logistic regression models after multiple imputation: power and applicability
@@ -64,22 +68,27 @@
 #'   status="Status", predictors=c("Duration", "Radiation", "Onset"), p.crit=1, 
 #'   method="D1", cat.predictors=c("Expect_cat"))
 #'   pool_coxr$RR_Model
-#'   pool_coxr$multiparm_p
+#'   pool_coxr$multiparm
 #'   
+#'  \dontrun{ 
 #'   pool_coxr <- psfmi_coxr(data=lbpmicox, nimp=5, impvar="Impnr", time="Time", 
 #'   status="Status", predictors=c("Previous",  "Radiation", "Onset",
 #'   "Function", "Tampascale" ), p.crit=0.05, cat.predictors=c("Expect_cat"), 
 #'   int.predictors=c("Tampascale:Radiation",
 #'   "Expect_cat:Tampascale"), keep.predictors = "Tampascale", method="D2")
 #'   pool_coxr$RR_Model
-#'   pool_coxr$multiparm_p
+#'   pool_coxr$multiparm
+#'   pool_coxr$predictors_in
+#'  } 
 #'   
 #' @export
 psfmi_coxr <-
   function(data, nimp=5, impvar=NULL, time, status, predictors=NULL,
    p.crit=1, cat.predictors=NULL, spline.predictors=NULL, int.predictors=NULL,
-   keep.predictors=NULL, knots=NULL, method=NULL, print.method=FALSE)
+   keep.predictors=NULL, knots=NULL, method="RR", print.method=FALSE)
   {
+    call <- match.call()
+    
     P <- predictors
     cat.P <- cat.predictors
     keep.P <- keep.predictors
@@ -88,15 +97,19 @@ psfmi_coxr <-
     P.check <-c(P, cat.P, s.P)
     
     # Check data input
-    if (!(is.matrix(data) | is.data.frame(data)))
-      stop("Data should be a matrix or data frame")
-    data <- data.frame(data.matrix(data))
+    if (!(is.data.frame(data)))
+      stop("Data should be a data frame")
+    data <- data.frame(as_tibble(data))
+    data <- mutate_if(data, is.factor, ~ as.numeric(as.character(.x)))
+    if(!all(data[status]==1 | data[status]==0))
+      stop("Outcome should be a 0 - 1 variable")
     if ((nvar <- ncol(data)) < 2)
       stop("Data should contain at least two columns")
     if(is.null(impvar))
       stop("Imputation variable is not defined")
-    if(is.null(method))
-      stop("Define selection method: D1, D2 or MPR")
+    if(is.null(method)) method="RR"
+    if(all(!is.null(cat.predictors) | !is.null(spline.predictors)) & method=="RR")
+      stop("Categorical or spline variables in model, define selection method: D1, D2, D3 or MPR")
     if (order(unique(data[, impvar]))[1] == 0)
       stop("Original dataset should not be included")
     if(is.null(nimp))
@@ -144,7 +157,7 @@ psfmi_coxr <-
              Predictor or Categorical Predictor", "\n\n")
     }
     
-    # First predictors, second cetegorical
+    # First predictors, second categorical
     # predictors and last interactions!
     P <- c(P, cat.P, s.P, int.P)
     if (is.null(P))
@@ -261,9 +274,9 @@ psfmi_coxr <-
     if(any(!keep.P %in% P))
       stop("\n", "Variables to keep not defined as Predictor", "\n\n")
     
-    coef.f <- se.f <- RR.model <- multiparm_p <- coef.excl_step <- step.nr <- list()
+    coef.f <- se.f <- RR.model <- multiparm <- coef.excl_step <- step.nr <- P_in_step <- list()
     for (k in 1:length(P)) {
-      
+      P_in_step[[k]] <- P
       chi.LR <- data.frame(matrix(0,
                                   length(P), nimp))
       chi.p <- data.frame(matrix(0,
@@ -326,7 +339,10 @@ psfmi_coxr <-
         stop("\n", "Check Pooled Model,
         some parameters could not be estimated", "\n")
       }
-      
+      if(method=="RR"){
+        p.pool <- data.frame(pool.RR[, 3])
+        multiparm <- NULL
+      }
       # D2
       if(method=="D2")
       {
@@ -348,15 +364,15 @@ psfmi_coxr <-
         id.p.RR.spl <- grep("rcs", row.names(pool.RR))
         res.RR <- pool.RR[-c(id.p.RR.f, id.p.RR.spl), 3]
         mi.chisq[names(res.RR), 2] <- res.RR
-        names(mi.chisq) <- c("D2", "D2 & RR p-values")
+        names(mi.chisq) <- c("D2", "p-value D2 & RR")
         
         if(print.method) {
           mi.chisq <- mi.chisq_orig
-          names(mi.chisq) <- c("D2", "D2 p-values")
+          names(mi.chisq) <- c("D2", "p-value D2")
         }
         
-        multiparm_p[[k]] <- mi.chisq
-        names(multiparm_p)[k] <- paste("Step", k)
+        multiparm[[k]] <- mi.chisq
+        names(multiparm)[k] <- paste("Step", k)
         # Res.f is the dataframe with the CHISQ
         # pooled p-values for all variables
       }
@@ -375,15 +391,15 @@ psfmi_coxr <-
         id.p.RR.spl <- grep("rcs", row.names(pool.RR))
         res.RR <- pool.RR[-c(id.p.RR.f, id.p.RR.spl), 3]
         est.D1[names(res.RR), 2] <- res.RR
-        names(est.D1) <- c("Chi_sq", "D1 & RR p-values")
+        names(est.D1) <- c("Chi_sq", "p-value D1 & RR")
         
         if(print.method) {
           est.D1 <- est.D1_orig
-          names(est.D1) <- c("Chi_sq", "D1 p-values")
+          names(est.D1) <- c("Chi_sq", "p-value D1")
         }
         
-        multiparm_p[[k]] <- est.D1
-        names(multiparm_p)[k] <- paste("Step", k)
+        multiparm[[k]] <- est.D1
+        names(multiparm)[k] <- paste("Step", k)
       }
       # Med P Rule
       if(method=="MPR")
@@ -391,7 +407,7 @@ psfmi_coxr <-
         med.pvalue <- med.pvalue_orig <- round(data.frame(apply(chi.p,
                                                                 1, median)), 4)
         row.names(med.pvalue) <- row.names(med.pvalue_orig) <- P
-        names(med.pvalue) <- "MPR & RR P-values"
+        names(med.pvalue) <- "P-value MPR & RR"
         
         # Combine Median p with RR
         id.p.RR.f <- grep("factor", row.names(pool.RR))
@@ -401,24 +417,24 @@ psfmi_coxr <-
         
         if(print.method) {
           med.pvalue <- med.pvalue_orig
-          names(med.pvalue_orig) <- "MPR P-values"
+          names(med.pvalue) <- "P-value MPR"
         }
         
-        multiparm_p[[k]] <- med.pvalue
-        names(multiparm_p)[k] <- paste("Step", k)
+        multiparm[[k]] <- med.pvalue
+        names(multiparm)[k] <- paste("Step", k)
       }
       
       if(method=="D2"){
         p.pool <- data.frame(as.matrix(mi.chisq)[, 2])
-        names(p.pool) <- "RR & D2 p-value"
+        names(p.pool) <- "p-value RR & D2"
       }
       if(method=="D1"){
         p.pool <- data.frame(as.matrix(est.D1)[, 2])
-        names(p.pool) <- "RR & D1 p-value"
+        names(p.pool) <- "p-value RR & D1"
       }
       if(method=="MPR"){
         p.pool <- med.pvalue
-        names(p.pool) <- "RR & MPR p-value"
+        names(p.pool) <- "p-value RR & MPR"
       }
       
       if(!is.null(int.P)) {
@@ -552,6 +568,7 @@ psfmi_coxr <-
       
       if (p.pool[, 1][del.coef.id] > p.crit) {
         if (length(P) == 0) {
+          coef.excl_step[[k]] <- coef.excl
           message("\n", "Selection correctly terminated, ",
                   "\n", "Model is empty after last step", "\n")
           (break)()
@@ -573,17 +590,49 @@ psfmi_coxr <-
       coef.excl_step[[k]] <- coef.excl
       # End k loop
     }
-    if(p.crit==1) coef.excl_step <- as.null(coef.excl_step)
-    else {
-      coef.excl_step <- data.frame(do.call("rbind", coef.excl_step))
-      names(coef.excl_step) <- "Excluded"
-      row.names(coef.excl_step) <- paste("Step", 1:nrow(coef.excl_step))
+    if(p.crit==1) {
+      coef.excl_step <- as.null(coef.excl_step)
+      P_select <- data.frame(matrix(1, 1, length(P_in_step[[1]]), byrow=TRUE))
+      colnames(P_select) <- P_in_step[[1]]
+      predictors_final <- P_in_step[[1]]
     }
-    
-    pooledobj <- list("RR_Model"=RR.model, "multiparm_p"=multiparm_p, "coef.excl_step" = coef.excl_step,
-      "impvar"=impvar, "nimp"=nimp, "method"=method, "p.crit"=p.crit,
-      "predictors"=predictors, "cat.predictors"=cat.predictors,
-      "keep.predictors"=keep.predictors, "int.predictors"=int.predictors,
-      "spline.predictors"=spline.predictors, "knots"=knots, "print.method"=print.method)
-    return(pooledobj)
+    else {
+      if(is_empty(coef.excl_step)){
+        coef.excl_step <- as.null(coef.excl_step)
+        P_select <- data.frame(matrix(1, 1, length(P_in_step[[1]]), byrow=TRUE))
+        rownames(P_select) <- "Step 1"
+        colnames(P_select) <- P_in_step[[1]]
+      }
+      else{  
+        coef.excl_step <- data.frame(do.call("rbind", coef.excl_step))
+        names(coef.excl_step) <- "Excluded"
+        row.names(coef.excl_step) <- paste("Step", 1:nrow(coef.excl_step))
+        
+        outOrder_step <- P_in_step[[1]]
+        P_select <- data.frame(do.call("rbind", lapply(P_in_step, function(x) {
+          outOrder_step %in% x
+        })))
+        names(P_select) <- P_in_step[[1]]
+        P_select[P_select==TRUE] <- 1
+        row.names(P_select) <- paste("Step", 1:nrow(P_select))
+        if(length(P_in_step[[1]]) == length(coef.excl_step[, 1])) {
+          r_null <- rep(0, length(P_in_step[[1]]))
+          names(r_null) <- P_in_step[[1]]
+          P_select <- bind_rows(P_select, r_null)
+          row.names(P_select) <- paste("Step", 1:nrow(P_select))
+        }
+      }
+      predictors_final <- colnames(P_select)[which(P_select[nrow(P_select), ]==1)]
+    }
+    fit.formula <- as.formula(paste(Y, paste(P_in_step[[1]], collapse = "+")))
+    pobj <- list(data = data, RR_Model = RR.model, multiparm = multiparm,
+        predictors_in = P_select, predictors_out = coef.excl_step, time = time,
+        status = status, impvar = impvar, nimp = nimp, method = method, p.crit = p.crit,
+        predictors = predictors, cat.predictors = cat.predictors, call = call,
+        keep.predictors = keep.predictors, int.predictors = int.predictors, model_type = "survival",
+        spline.predictors = spline.predictors, knots = knots, print.method = print.method,
+        fit.formula = fit.formula, predictors_final = predictors_final,
+        predictors_initial = P_in_step[[1]])
+    class(pobj) <- "smodsmi"
+    return(pobj)
 }
